
//*************************************************************************
//  POV-Blur Motion-Blur Script v0.9
//  (c) 1999 By Chris Hernandez.
//*************************************************************************


//-------------------------------------------------------------------------
// STANDARD OPTIONS.
//-------------------------------------------------------------------------

   #declare Input_File_Name  = "sample.pov"
   #declare Output_File_Name = "output"

   #declare Start_Clock = 0.00;
   #declare End_Clock   = 1.00;
   
   #declare Start_Frame = 1;
   #declare End_Frame   = 10;
   
   #declare SamplesPerFrame = 7;
   #declare BlurLength = 0.65;
   
   #declare DisplayWhileDoingFrames = false;
   #declare DisplayWhileDoingBlur   = true;


//-------------------------------------------------------------------------
// ADVANCED OPTIONS.
//-------------------------------------------------------------------------
   
   #declare BlurFirstFrame = true;
   #declare BlurLastFrame  = true;
   
   #declare TempFileName        = "temp"
   #declare F2CincFileName      = "FrmToClk.inc"
   #declare DoFramesFileName    = "DoFrames.pov"
   #declare DoFramesIniFileName = "DoFrames.ini"
   #declare DoBlurFileName      = "DoBlur.pov"
   #declare DoBlurIniFileName   = "DoBlur.ini"
   
   #declare F2CArrayName      = "FrameToClockMapper"
   #declare ClockVariableName = "PBclock"



//-------------------------------------------------------------------------
// ***EXPERIMENTAL OPTIONS***
//   These options are experimental at this stage. I suggest leaving
//   UseOverlap in automatic mode.
//-------------------------------------------------------------------------

   #declare AUTO = 3;
   #declare UseOverlap = AUTO;
   #declare UseDefaultOverlapFrames = true;
   #declare OverlapFrames = 2;



// ------- Do math for scene.

   #declare BaseClk = 0;
   
   #if (Start_Frame != End_Frame)
      #declare Clk_Delta = (End_Clock-Start_Clock)/((End_Frame-Start_Frame));
   #else
      #declare Clk_Delta = End_Clock-Start_Clock;
   #end
   
   #if (UseOverlap = true)
      #if (UseDefaultOverlapFrames = true)
         #declare OverlapFrames = 1;
      #end
   #else
      #declare OverlapFrames = 0;
   #end
   
   #if (SamplesPerFrame != 1)
      #declare Frame_Delta = (Clk_Delta/(SamplesPerFrame-1))*BlurLength;
      #declare Frame_Offset = (0.5*(Clk_Delta*BlurLength));
      #declare TotalFrames = SamplesPerFrame + ((End_Frame-Start_Frame)*(SamplesPerFrame-OverlapFrames));
   #else
      #declare Frame_Delta = Clk_Delta;
      #declare Frame_Offset = 0;
      #declare TotalFrames = (End_Frame-Start_Frame)+1;
   #end
   
   #declare Frame_Increment = Frame_Delta*BlurLength;
   
   
   
   

// ------- Generate frame -> Clock mapping file.

   #fopen F2CFile F2CincFileName write
   
   #write(F2CFile, "//***********************************************************************\n")
   #write(F2CFile, "//*  FRAME TO CLOCK MAPPER FILE                                         *\n")
   #write(F2CFile, "//*    This file has been automatically generated using the POV-Blur    *\n")
   #write(F2CFile, "//*    motion blur script (c) 1999 Chris Hernandez.                     *\n")
   #write(F2CFile, "//***********************************************************************\n")
   #write(F2CFile, "\n")
   #write(F2CFile, "//                               ---<*>---                               \n")
   #write(F2CFile, "//   It is strongly advised that you do not manually modify the          \n")
   #write(F2CFile, "//   contents of this file. Doing so may cause the other automatically   \n")
   #write(F2CFile, "//   generated scripts to malfunction... You have been warned!           \n")
   #write(F2CFile, "//                               ---<*>---                               \n")
   #write(F2CFile, "\n")
   #write(F2CFile, "\n")
   
   #write(F2CFile, "#declare IMPORT_POVBLUR = true;\n")
   #write(F2CFile, "\n")
   
   // --- Initialize the array which contains the mapping
   #write(F2CFile, concat("#declare ", F2CArrayName, "= array[", str(TotalFrames+1,0,0),"]\n"))
   #write(F2CFile, "\n")
   
   #local FrameCounter = 1;
   #local TempFrameCounter = 1;
   
   //#write(F2CFile, concat(str(Clk_Delta, 5, 5), " \n"))
   //#write(F2CFile, concat(str(Frame_Delta, 5, 5), " \n"))
   
   #while (FrameCounter <= ((End_Frame-Start_Frame)+1))
      
      //#debug concat(str(FrameCounter,0,0)," FRAMESTART - \n")
      
      #local FrameClock = Start_Clock + ((FrameCounter-1)*Clk_Delta);
      
      #local FrameStart = FrameClock - Frame_Offset;
      
      //#write(F2CFile, concat(str(FrameStart, 5, 5), " "))
      //#write(F2CFile, concat(str(FrameClock, 5, 5), " "))
      //#write(F2CFile, concat(str(Frame_Offset, 5, 5), " "))
      
      #write(F2CFile, concat("\n// Samples for frame ", str(FrameCounter,0,0), "\n"))
      
      #if (FrameCounter=1)
      
         #local FrameLoop = 1;
         #while (FrameLoop <= SamplesPerFrame)
            //#debug concat(str(FrameLoop,0,0),"     Hi!\n")
            #local ClockVal = FrameStart + (FrameLoop-1)*Frame_Delta;
            #write(F2CFile, concat("#declare ", F2CArrayName, "[", str(TempFrameCounter,0,0),"] = ", str(ClockVal, 5, 5),";\n"))
            #local TempFrameCounter = TempFrameCounter+1;
            #local FrameLoop = FrameLoop+1;
         #end
     
      #else
      
         #local FrameLoop = OverlapFrames+1;
         #while (FrameLoop <= SamplesPerFrame)
            //#debug concat(str(FrameLoop,0,0),"     Hi!\n")
            #local ClockVal = FrameStart + (FrameLoop-1)*Frame_Delta;
            #write(F2CFile, concat("#declare ", F2CArrayName, "[", str(TempFrameCounter,0,0),"] = ", str(ClockVal, 5, 5),";\n"))
            #local TempFrameCounter = TempFrameCounter+1;
            #local FrameLoop = FrameLoop+1;
         #end

      #end
      
      #local FrameCounter = FrameCounter + 1;
   
   #end
   
   #fclose F2CFile
      


// ------- Generate DoFrames.pov

   #fopen DoFramesFile DoFramesFileName write
   
   #write(DoFramesFile, "//***********************************************************************\n")
   #write(DoFramesFile, "//*  MOTION BLUR FRAME(S) GENERATOR SOURCE FILE                         *\n")
   #write(DoFramesFile, "//*    This file has been automatically generated using the POV-Blur    *\n")
   #write(DoFramesFile, "//*    motion blur script (c) 1999 Chris Hernandez.                     *\n")
   #write(DoFramesFile, "//***********************************************************************\n")
   #write(DoFramesFile, "\n")
   #write(DoFramesFile, "//                               ---<*>---                               \n")
   #write(DoFramesFile, "//   It is strongly advised that you do not manually modify the          \n")
   #write(DoFramesFile, "//   contents of this file. Doing so may cause the other automatically   \n")
   #write(DoFramesFile, "//   generated scripts to malfunction... You have been warned!           \n")
   #write(DoFramesFile, "//                               ---<*>---                               \n")
   #write(DoFramesFile, "\n")
   #write(DoFramesFile, "\n")
   
   #write(DoFramesFile, "#declare IMPORT_POVBLUR = true;\n")
   #write(DoFramesFile, "\n")
   
   #write(DoFramesFile, concat("#include \"", F2CincFileName,"\"\n"))
   #write(DoFramesFile, "\n")
   
   #write(DoFramesFile, concat("#declare ", ClockVariableName, " = ", F2CArrayName, "[clock]", ";\n"))
   #write(DoFramesFile, "\n")
   
   #write(DoFramesFile, concat("#include \"", Input_File_Name,"\"\n"))
   #write(DoFramesFile, "\n")
   
   #write(DoFramesFile, concat("#render concat(\"\nRendering frame \", str(clock,0,0), \" of ", str(TotalFrames,0,0),"\n\")\n"))

   #fclose DoFramesFile



// ------- Generate DoFrames.ini
   
   #fopen DoFramesIniFile DoFramesIniFileName write
   
   #write(DoFramesIniFile, ";***********************************************************************\n")
   #write(DoFramesIniFile, ";*  MOTION BLUR FRAME(S) GENERATOR INI FILE                            *\n")
   #write(DoFramesIniFile, ";*    This file has been automatically generated using the POV-Blur    *\n")
   #write(DoFramesIniFile, ";*    motion blur script (c) 1999 Chris Hernandez.                     *\n")
   #write(DoFramesIniFile, ";***********************************************************************\n")
   #write(DoFramesIniFile, "\n")
   #write(DoFramesIniFile, ";                               ---<*>---                               \n")
   #write(DoFramesIniFile, ";   It is strongly advised that you do not manually modify the          \n")
   #write(DoFramesIniFile, ";   contents of this file. Doing so may cause the other automatically   \n")
   #write(DoFramesIniFile, ";   generated scripts to malfunction... You have been warned!           \n")
   #write(DoFramesIniFile, ";                               ---<*>---                               \n")
   #write(DoFramesIniFile, "\n")
   #write(DoFramesIniFile, "\n")
   
   #write(DoFramesIniFile, concat("Input_File_Name = ", DoFramesFileName, "\n"))
   #write(DoFramesIniFile, concat("Output_File_Name = ", TempFileName, "\n"))
   #write(DoFramesIniFile, "\n")
   #write(DoFramesIniFile, concat("Initial_Frame = 1\n"))
   #write(DoFramesIniFile, concat("Final_Frame = ", str(TotalFrames,0,0), "\n"))
   #write(DoFramesIniFile, "\n")
   #write(DoFramesIniFile, "Initial_Clock = 1\n")
   #write(DoFramesIniFile, concat("Final_Clock = ", str(TotalFrames, 4, 4), "\n"))
   #write(DoFramesIniFile, "\n")
   #write(DoFramesIniFile, "Output_file_type= t\n")
   #write(DoFramesIniFile, concat("Display = ", str(DisplayWhileDoingFrames,1,1), "\n"))
  
   #fclose DoFramesIniFile

// ------- Generate DoBlur.pov

   #fopen DoBlurFile DoBlurFileName write
   
   #write(DoBlurFile, "//***********************************************************************\n")
   #write(DoBlurFile, "//*  MOTION BLUR FRAME INTEGRATION SOURCE FILE                          *\n")
   #write(DoBlurFile, "//*    This file has been automatically generated using the POV-Blur    *\n")
   #write(DoBlurFile, "//*    motion blur script (c) 1999 Chris Hernandez.                     *\n")
   #write(DoBlurFile, "//***********************************************************************\n")
   #write(DoBlurFile, "\n")
   #write(DoBlurFile, "//                               ---<*>---                               \n")
   #write(DoBlurFile, "//   It is strongly advised that you do not manually modify the          \n")
   #write(DoBlurFile, "//   contents of this file. Doing so may cause the other automatically   \n")
   #write(DoBlurFile, "//   generated scripts to malfunction... You have been warned!           \n")
   #write(DoBlurFile, "//                               ---<*>---                               \n")
   #write(DoBlurFile, "\n")
   #write(DoBlurFile, "\n")
   
   #local FramesToBlur = (End_Frame - Start_Frame) + 1;
   #local FrameCounter = 1;
   #local FrameStringLength = strlen(str(TotalFrames,0,0));
   #local SampleStringLength = strlen(str(SamplesPerFrame,0,0));
   
   //define layers
   //define texturing.
   
   #write(DoBlurFile, "\n")
   #write(DoBlurFile, concat("#declare BaseFrame = ((clock-1)*", str(((SamplesPerFrame-OverlapFrames)),0,0),")+1;\n"))
   #write(DoBlurFile, "\n")
   
   #local SampleCounter = 1;   
   #while (SampleCounter <= SamplesPerFrame)
 
      #write(DoBlurFile, concat("#declare Frame", str(SampleCounter, -SampleStringLength, 0),
                                " = concat(\"", TempFileName, "\", str(BaseFrame+",str(SampleCounter-1,0,0),
                                ",-", str(FrameStringLength,0,0), ",0), \".tga\")\n"))
   
      #local SampleCounter = SampleCounter + 1;
      
   #end
   
   #write(DoBlurFile, "\n")
   
   #local SampleCounter = 1;
   #while (SampleCounter <= SamplesPerFrame)
 
      #write(DoBlurFile, concat("#declare layer", str(SampleCounter, -SampleStringLength, 0),
                                " = pigment { image_map { tga Frame", str(SampleCounter, -SampleStringLength, 0),
                                " interpolate 2 map_type 0 } }\n"))
   
      #local SampleCounter = SampleCounter + 1;
      
   #end
   
      
   #write(DoBlurFile, "\ncamera {location <0, 0, -1> look_at <0, 0, 0> }\n")
   #write(DoBlurFile, "\n")
   #write(DoBlurFile, "\n")
   
   #write(DoBlurFile, "box {<0, 0, 0>, <1, 1, 1>\n")
   #write(DoBlurFile, "   texture {\n")
   #write(DoBlurFile, "      pigment {\n")
   #write(DoBlurFile, "         average\n")
   #write(DoBlurFile, "         pigment_map {\n")
   
   #local SampleCounter = 1;
   #while (SampleCounter <= SamplesPerFrame)
 
      #write(DoBlurFile, concat("            [1 layer", str(SampleCounter, -SampleStringLength, 0), " ]\n"))
   
      #local SampleCounter = SampleCounter + 1;
      
   #end
   
   #write(DoBlurFile, "         }\n")
   #write(DoBlurFile, "      }\n")
   #write(DoBlurFile, "      finish { ambient 1.0  diffuse 0.1 }\n")
   #write(DoBlurFile, "   }\n")
   #write(DoBlurFile, "   translate <-.5, -.5, 0>\n")
   #write(DoBlurFile, "   scale <4/3, 1, 1>\n")
   #write(DoBlurFile, "}\n"
   
   
   #fclose DoBlurFile

// ------- Generate DoBlur.ini


   #fopen DoBlurIniFile DoBlurIniFileName write
   
   #write(DoBlurIniFile, ";***********************************************************************\n")
   #write(DoBlurIniFile, ";*  MOTION BLUR FRAME INTEGRATION INI FILE                             *\n")
   #write(DoBlurIniFile, ";*    This file has been automatically generated using the POV-Blur    *\n")
   #write(DoBlurIniFile, ";*    motion blur script (c) 1999 Chris Hernandez.                     *\n")
   #write(DoBlurIniFile, ";***********************************************************************\n")
   #write(DoBlurIniFile, "\n")
   #write(DoBlurIniFile, ";                               ---<*>---                               \n")
   #write(DoBlurIniFile, ";   It is strongly advised that you do not manually modify the          \n")
   #write(DoBlurIniFile, ";   contents of this file. Doing so may cause the other automatically   \n")
   #write(DoBlurIniFile, ";   generated scripts to malfunction... You have been warned!           \n")
   #write(DoBlurIniFile, ";                               ---<*>---                               \n")
   #write(DoBlurIniFile, "\n")
   #write(DoBlurIniFile, "\n")
   
   #write(DoBlurIniFile, concat("Input_File_Name = ", DoBlurFileName, "\n"))
   #write(DoBlurIniFile, concat("Output_File_Name = ", Output_File_Name, "\n"))
   #write(DoBlurIniFile, "\n")
   #write(DoBlurIniFile, concat("Initial_Frame = 1\n"))
   #write(DoBlurIniFile, concat("Final_Frame = ", str(FramesToBlur,0,0), "\n"))
   #write(DoBlurIniFile, "\n")
   #write(DoBlurIniFile, "Initial_Clock = 1\n")
   #write(DoBlurIniFile, concat("Final_Clock = ", str(FramesToBlur, 4, 4), "\n"))
   #write(DoBlurIniFile, "\n")
   #write(DoBlurIniFile, "Output_file_type= t\n")
   #write(DoBlurIniFile, concat("Display = ", str(DisplayWhileDoingBlur,1,1), "\n")))
  
   #fclose DoBlurIniFile
   

// Dummy object

   camera { location <0, 0, -5> look_at <0,0,0> angle 35 }

   light_source { <-5, 7, -5> color rgb 0.95 }

   sphere {
      0.1, 0.3
      texture { 
         pigment{ color rgb<1.0, 0.15, 0.1> }
         finish { ambient 0.3 diffuse 0.3 specular 0.5 roughness 0.025 }
      }
      translate y*0.75
   }
   
   sphere {
      0.1, 0.3
      texture { 
         pigment{ color rgb<1.0, 0.95, 0.1> }
         finish { ambient 0.3 diffuse 0.3 specular 0.15 roughness 0.025 }
      }
      translate y*0.0
   }
   
   sphere {
      0.1, 0.3
      texture { 
         pigment{ color rgb<0.1, 0.95, 0.15> }
         finish { ambient 0.3 diffuse 0.3 specular 0.15 roughness 0.025 }
      }
      translate -y*0.75
   }
   